@@ -1,9 +1,9 @@
 # coding=utf-8
 #
 # This file is part of Hypothesis, which may be found at
-# https://github.com/HypothesisWorks/hypothesis-python
+# https://github.com/HypothesisWorks/hypothesis/
 #
-# Most of this work is copyright (C) 2013-2018 David R. MacIver
+# Most of this work is copyright (C) 2013-2019 David R. MacIver
 # (david@drmaciver.com), but it contains contributions by others. See
 # CONTRIBUTING.rst for a full list of people who may hold copyright, and
 # consult the git log if you need to determine who owns an individual
@@ -11,7 +11,7 @@
 #
 # This Source Code Form is subject to the terms of the Mozilla Public License,
 # v. 2.0. If a copy of the MPL was not distributed with this file, You can
-# obtain one at http://mozilla.org/MPL/2.0/.
+# obtain one at https://mozilla.org/MPL/2.0/.
 #
 # END HEADER
 
@@ -27,11 +27,9 @@ import inspect
 import re
 import tokenize
 import types
-import uuid
 from functools import wraps
 from types import ModuleType
 
-from hypothesis.configuration import storage_directory
 from hypothesis.internal.compat import (
     ARG_NAME_ATTRIBUTE,
     getfullargspec,
@@ -56,17 +54,15 @@ def fully_qualified_name(f):
 
 
 def is_mock(obj):
-    """Determine if the given argument is a mock type.
+    """Determine if the given argument is a mock type."""
 
-    We want to be able to detect these when dealing with various test
-    args. As they are sneaky and can look like almost anything else,
-    we'll check this by looking for random attributes.  This is more
-    robust than looking for types.
-    """
-    for _ in range(10):
-        if not hasattr(obj, str(uuid.uuid4())):
-            return False
-    return True
+    # We want to be able to detect these when dealing with various test
+    # args. As they are sneaky and can look like almost anything else,
+    # we'll check this by looking for an attribute with a name that it's really
+    # unlikely to implement accidentally, and that anyone who implements it
+    # deliberately should know what they're doing. This is more robust than
+    # looking for types.
+    return hasattr(obj, "hypothesis_internal_is_this_a_mock_check")
 
 
 def function_digest(function):
@@ -182,7 +178,7 @@ def convert_keyword_arguments(function, args, kwargs):
             else:
                 raise TypeError("No value provided for argument %r" % (arg_name))
 
-    if kwargs and not argspec.varkw:
+    if kwargs and not (argspec.varkw or argspec.kwonlyargs):
         if len(kwargs) > 1:
             raise TypeError(
                 "%s() got unexpected keyword arguments %s"
@@ -294,7 +290,10 @@ def extract_lambda_source(f):
         else:
             arg_strings.append(a)
 
-    if_confused = "lambda %s: <unknown>" % (", ".join(arg_strings),)
+    if arg_strings:
+        if_confused = "lambda %s: <unknown>" % (", ".join(arg_strings),)
+    else:
+        if_confused = "lambda: <unknown>"
     if bad_lambda:  # pragma: no cover
         return if_confused
     try:
@@ -374,9 +373,9 @@ def extract_lambda_source(f):
     # AttributeError.
     #
     try:
-        encoding, _ = tokenize.detect_encoding(
-            open(inspect.getsourcefile(f), "rb").readline
-        )
+        with open(inspect.getsourcefile(f), "rb") as src_f:
+            encoding, _ = tokenize.detect_encoding(src_f.readline)
+
         source_bytes = source.encode(encoding)
         source_bytes = source_bytes[lambda_ast.col_offset :].strip()
         source = source_bytes.decode(encoding)
@@ -424,8 +423,7 @@ def get_pretty_function_description(f):
         return repr(f)
     name = f.__name__
     if name == "<lambda>":
-        result = extract_lambda_source(f)
-        return result
+        return extract_lambda_source(f)
     elif isinstance(f, types.MethodType):
         self = f.__self__
         if not (self is None or inspect.isclass(self)):
@@ -471,10 +469,6 @@ def check_valid_identifier(identifier):
         raise ValueError("%r is not a valid python identifier" % (identifier,))
 
 
-def eval_directory():
-    return storage_directory("eval_source")
-
-
 eval_cache = {}  # type: dict
 
 
@@ -494,17 +488,14 @@ def source_exec_as_module(source):
     return result
 
 
-COPY_ARGSPEC_SCRIPT = (
-    """
+COPY_ARGSPEC_SCRIPT = """
 from hypothesis.utils.conventions import not_set
 
 def accept(%(funcname)s):
     def %(name)s(%(argspec)s):
         return %(funcname)s(%(invocation)s)
     return %(name)s
-""".strip()
-    + "\n"
-)
+""".lstrip()
 
 
 def define_function_signature(name, docstring, argspec):
@@ -617,7 +608,9 @@ def proxies(target):
         return impersonate(target)(
             wraps(target)(
                 define_function_signature(
-                    target.__name__, target.__doc__, getfullargspec(target)
+                    target.__name__.replace("<lambda>", "_lambda_"),
+                    target.__doc__,
+                    getfullargspec(target),
                 )(proxy)
             )
         )
