@@ -43,7 +43,7 @@ from hypothesis._settings import (
 from hypothesis.control import current_build_context
 from hypothesis.core import given
 from hypothesis.errors import InvalidArgument, InvalidDefinition
-from hypothesis.internal.compat import int_to_bytes, string_types
+from hypothesis.internal.compat import quiet_raise, string_types
 from hypothesis.internal.reflection import function_digest, nicerepr, proxies
 from hypothesis.internal.validation import check_type
 from hypothesis.reporting import current_verbosity, report
@@ -89,7 +89,17 @@ def run_state_machine_as_test(state_machine_factory, settings=None):
     @given(st.data())
     def run_state_machine(factory, data):
         machine = factory()
-        check_type(GenericStateMachine, machine, "state_machine_factory()")
+        if isinstance(machine, GenericStateMachine) and not isinstance(
+            machine, RuleBasedStateMachine
+        ):
+            note_deprecation(
+                "%s inherits from GenericStateMachine, which is deprecated.  Use a "
+                "RuleBasedStateMachine, or a test function with st.data(), instead."
+                % (type(machine).__name__,),
+                since="2019-05-29",
+            )
+        else:
+            check_type(RuleBasedStateMachine, machine, "state_machine_factory()")
         data.conjecture_data.hypothesis_runner = machine
 
         n_steps = settings.stateful_step_count
@@ -150,25 +160,28 @@ class GenericStateMachineMeta(type):
 class GenericStateMachine(
     GenericStateMachineMeta("GenericStateMachine", (object,), {})  # type: ignore
 ):
-    """A GenericStateMachine is the basic entry point into Hypothesis's
-    approach to stateful testing.
+    """A GenericStateMachine is a deprecated approach to stateful testing.
 
-    The intent is for it to be subclassed to provide state machine descriptions
+    In earlier versions of Hypothesis, you would define ``steps``,
+    ``execute_step``, ``teardown``, and ``check_invariants`` methods;
+    and the engine would then run something like the following::
 
-    The way this is used is that Hypothesis will repeatedly execute something
-    that looks something like::
+        @given(st.data())
+        def test_the_stateful_thing(data):
+            x = MyStatemachineSubclass()
+            x.check_invariants()
+            try:
+                for _ in range(50):
+                    step = data.draw(x.steps())
+                    x.execute_step(step)
+                    x.check_invariants()
+            finally:
+                x.teardown()
 
-        x = MyStatemachineSubclass()
-        x.check_invariants()
-        try:
-            for _ in range(n_steps):
-                x.execute_step(x.steps().example())
-                x.check_invariants()
-        finally:
-            x.teardown()
-
-    And if this ever produces an error it will shrink it down to a small
-    sequence of example choices demonstrating that.
+    We now recommend using rule-based stateful testing instead wherever
+    possible.  If your test is better expressed in the above format than
+    as a rule-based state machine, we suggest "unrolling" your method
+    definitions into a simple test function with the above control flow.
     """
 
     def steps(self):
@@ -206,11 +219,9 @@ class GenericStateMachine(
 
         Does nothing by default.
         """
-        pass
 
     def check_invariants(self):
         """Called after initializing and after executing each step."""
-        pass
 
     _test_case_cache = {}  # type: dict
 
@@ -595,42 +606,10 @@ class RuleStrategy(SearchStrategy):
         )
 
     def do_draw(self, data):
-        # This strategy is slightly strange in its implementation.
-        # We don't want the interpretation of the rule we draw to change based
-        # on whether other rules satisfy their preconditions or have data in
-        # their bundles. Therefore the index into the rule list needs to stay
-        # stable. BUT we don't want to draw invalid rules. So what we do is we
-        # draw an index. We *could* just loop until it's valid, but if most
-        # rules are invalid then that could result in a very long loop.
-        # So what we do is the following:
-        #
-        #   1. We first draw a rule unconditionally, and check if it's valid.
-        #      If it is, great. Nothing more to do, that's our rule.
-        #   2. If it is invalid, we now calculate the list of valid rules and
-        #      draw from that list (if there are none, that's an error in the
-        #      definition of the machine and we complain to the user about it).
-        #   3. Once we've drawn a valid rule, we write that back to the byte
-        #      stream. As a result, when shrinking runs the shrinker can delete
-        #      the initial failed draw + the draw that lead to us finding an
-        #      index into valid_rules, leaving just the written value of i.
-        #      When this is run, it will look as we got lucky and just happened
-        #      to pick a valid rule.
-        #
-        # Easy, right?
-        n = len(self.rules)
-        i = cu.integer_range(data, 0, n - 1)
-        u, v = data.blocks[-1].bounds
-        block_length = v - u
-        rule = self.rules[i]
-        if not self.is_valid(rule):
-            valid_rules = [j for j, r in enumerate(self.rules) if self.is_valid(r)]
-            if not valid_rules:
-                raise InvalidDefinition(
-                    u"No progress can be made from state %r" % (self.machine,)
-                )
-            i = valid_rules[cu.integer_range(data, 0, len(valid_rules) - 1)]
-            data.write(int_to_bytes(i, block_length))
-            rule = self.rules[i]
+        if not any(self.is_valid(rule) for rule in self.rules):
+            msg = u"No progress can be made from state %r" % (self.machine,)
+            quiet_raise(InvalidDefinition(msg))
+        rule = data.draw(st.sampled_from(self.rules).filter(self.is_valid))
         return (rule, data.draw(rule.arguments_strategy))
 
     def is_valid(self, rule):
